/**
 * @file cstrlist.h
 * @brief Zero separated string list.
 *
 * Copyright 2022 Raymond Czerny
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef CSTRLIST_H
#define CSTRLIST_H

#include <stdbool.h>
#include <stdint.h>
#include <wchar.h>

// === String list with Single-Byte-Characters ===========

// Structure to managment a Single-Byte-String list */
typedef struct
{
    char*   begin;  /**< Pointer of begin of list */
    char*   end;    /**< Pointer after memory */
} CStrList;

/**
 * @brief Function to allocate memory of string list on heap
 * @param list  Pointer to list
 * @param count Count of charaters and seperators
 * @return False if the memory is full, otherwise True
 */
bool cstrlist_create(CStrList* list, size_t count);

/**
 * @brief Free memory of string list on heap
 * @param list Pointer to list
 */
void cstrlist_destroy(CStrList* list);

/**
 * @brief Clean content of list
 * @param list Pointer to list
 */
void cstrlist_clean(CStrList* list);

/**
 * @brief Append a string to list
 * @param list Pointer to list
 * @param item New entry for list
 * @return Success
 */
bool cstrlist_append(CStrList* list, const char* item);

/**
 * @brief Append a string to the list without duplicates (case sensitive)
 * @param list Pointer to list
 * @param item New entry for list
 * @return
 */
bool cstrlist_append_unique(CStrList* list, const char* item);

/**
 * @brief Append a string to the list without duplicates (case insensitive)
 * @param list Pointer to list
 * @param item New entry for list
 * @return
 */
bool cstrlist_append_unique_i(CStrList* list, const char* item);

/**
 * @brief Remove entry from the list referenced by index
 * @param list Pointer to list
 * @param index Index of entry
 * @return Success
 */
bool cstrlist_remove(CStrList* list, size_t index);

/**
 * @brief Gets entry from the list referenced by index
 * @param list Pointer to list
 * @param index Index of entry
 * @return
 */
const char* cstrlist_value(CStrList* list, size_t index);

/**
 * @brief Get count of entries in list
 * @param list Pointer to list
 * @return Count of entries
 */
size_t cstrlist_count(CStrList* list);

/**
 * @brief Gets index of entry
 * @param list Pointer to list
 * @param item Search string (case sensitive)
 * @return Index on success, otherwise SIZE_MAX
 */
size_t cstrlist_index(CStrList* list, const char* item);

/**
 * @brief Gets index of entry
 * @param list Pointer to list
 * @param item Search string (case insensitive)
 * @return Index on success, otherwise SIZE_MAX
 */
size_t cstrlist_index_i(CStrList* list, const char* item);

/**
 * @brief Gets the next string from the list.
 * @param item Current string
 * @return First list entry if item is NULL,
 *         otherwise next entry or NULL at the end
 */
char* cstrlist_next(CStrList* list, char* item);

// === String list with Wide-Characters ===========

/** Structure to managment a Wide-Characters-String list */
typedef struct
{
    wchar_t*   begin;   /**< Pointer of begin of list */
    wchar_t*   end;     /**< Pointer after memory */
} WCStrList;

/**
 * @brief Function to allocate memory of string list on heap
 * @param list  Pointer to list
 * @param count Count of charaters and seperators
 * @return False if the memory is full, otherwise True
 */
bool wcstrlist_create(WCStrList* list, size_t count);

/**
 * @brief Free memory of string list on heap
 * @param list Pointer to list
 */
void wcstrlist_destroy(WCStrList* list);

/**
 * @brief Clean content of list
 * @param list Pointer to list
 */
void wcstrlist_clean(WCStrList* list);

/**
 * @brief Append a string to list
 * @param list Pointer to list
 * @param item New entry for list
 * @return Success
 */
bool wcstrlist_append(WCStrList* list, const wchar_t* item);

/**
 * @brief Append a string to the list without duplicates (case sensitive)
 * @param list Pointer to list
 * @param item New entry for list
 * @return
 */
bool wcstrlist_append_unique(WCStrList* list, const wchar_t* item);

/**
 * @brief Append a string to the list without duplicates (case insensitive)
 * @param list Pointer to list
 * @param item New entry for list
 * @return
 */
bool wcstrlist_append_unique_i(WCStrList* list, const wchar_t* item);

/**
 * @brief Remove entry from the list referenced by index
 * @param list Pointer to list
 * @param index Index of entry
 * @return Success
 */
bool wcstrlist_remove(WCStrList* list, size_t index);

/**
 * @brief Gets entry from the list referenced by index
 * @param list Pointer to list
 * @param index Index of entry
 * @return
 */
const wchar_t* wcstrlist_value(WCStrList* list, size_t index);

/**
 * @brief Get count of entries in list
 * @param list Pointer to list
 * @return Count of entries
 */
size_t wcstrlist_count(WCStrList* list);

/**
 * @brief Gets index of entry
 * @param list Pointer to list
 * @param item Search string (case sensitive)
 * @return Index on success, otherwise SIZE_MAX
 */
size_t wcstrlist_index(WCStrList* list, const wchar_t* item);

/**
 * @brief Gets index of entry
 * @param list Pointer to list
 * @param item Search string (case insensitive)
 * @return Index on success, otherwise SIZE_MAX
 */
size_t wcstrlist_index_i(WCStrList* list, const wchar_t* item);

/**
 * @brief Gets the next string from the list.
 * @param item Current string
 * @return First list entry if item is NULL,
 *         otherwise next entry or NULL at the end
 */
wchar_t* wcstrlist_next(WCStrList* list, wchar_t *item);


#endif // CSTRLIST_H


